"use strict";
// Copyright (c) 2020 Brandon Lehmann
//
// Please see the included LICENSE file for more information.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MySQL = void 0;
const events_1 = require("events");
const Types_1 = require("./Types");
const mysql_1 = require("mysql");
/**
 * MYSQL interface that implements IDatabase
 */
class MySQL extends events_1.EventEmitter {
    /**
     * Creates a new instance of the class
     * @param host
     * @param port
     * @param user
     * @param password
     * @param database
     * @param connectionLimit
     */
    constructor(host, port, user, password, database, connectionLimit = 10) {
        super();
        this.m_tableOptions = 'ENGINE=InnoDB PACK_KEYS=1 ROW_FORMAT=COMPRESSED';
        this.m_db = mysql_1.createPool({ host, port, user, password, database, connectionLimit });
        this.m_db.on('error', error => this.emit('error', error));
        this.m_db.on('acquire', connection => this.emit('acquire', connection));
        this.m_db.on('connection', connection => this.emit('connection', connection));
        this.m_db.on('enqueue', () => this.emit('enqueue'));
        this.m_db.on('release', connection => this.emit('release', connection));
        this.emit('ready');
    }
    get hashType() {
        return 'char(64)';
    }
    get blobType() {
        return 'text';
    }
    get uint32Type() {
        return 'int(10) unsigned';
    }
    get uint64Type() {
        return 'bigint(20) unsigned';
    }
    get tableOptions() {
        return this.m_tableOptions;
    }
    set tableOptions(value) {
        this.m_tableOptions = value;
    }
    get type() {
        return Types_1.Interfaces.DBType.MYSQL;
    }
    on(event, listener) {
        return super.on(event, listener);
    }
    createTable(name, fields, primaryKey, tableOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            const preparedTable = this.prepareCreateTable(name, fields, primaryKey, tableOptions);
            yield this.transaction([
                { query: preparedTable.table }
            ]);
            if (preparedTable.indexes.length !== 0)
                try {
                    const stmts = preparedTable.indexes.map(idx => {
                        return { query: idx };
                    });
                    yield this.transaction(stmts);
                }
                catch (error) {
                    this.emit('error', error);
                }
        });
    }
    prepareCreateTable(name, fields, primaryKey, tableOptions) {
        return Types_1.prepareCreateTable(this.type, name, fields, primaryKey, tableOptions);
    }
    close() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                this.m_db.end(err => {
                    if (err)
                        return reject(err);
                    return resolve();
                });
            });
        });
    }
    query(query, values) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                this.m_db.query(query, values || [], (error, results) => {
                    if (error)
                        return reject(error);
                    const count = results.changedRows ||
                        results.affectedRows ||
                        results.insertId ||
                        results.length;
                    return resolve([count, results]);
                });
            });
        });
    }
    transaction(queries) {
        return __awaiter(this, void 0, void 0, function* () {
            const connection = yield this.connection();
            try {
                yield beginTransaction(connection);
                for (const q of queries) {
                    yield query(connection, q.query, q.values);
                }
                yield commit(connection);
            }
            catch (e) {
                yield rollback(connection);
                throw e;
            }
            finally {
                connection.release();
            }
        });
    }
    prepareMultiInsert(query, values) {
        if (values) {
            const escaped = mysql_1.escape(values);
            return query.replace('%L', escaped);
        }
        return query;
    }
    connection() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                this.m_db.getConnection((error, connection) => {
                    if (error)
                        return reject(error);
                    return resolve(connection);
                });
            });
        });
    }
}
exports.MySQL = MySQL;
/** @ignore */
function beginTransaction(connection) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            connection.beginTransaction(error => {
                if (error)
                    return reject(error);
                return resolve();
            });
        });
    });
}
/** @ignore */
function commit(connection) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            connection.commit(error => {
                if (error)
                    return reject(error);
                return resolve();
            });
        });
    });
}
/** @ignore */
function query(connection, query, values = []) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            connection.query(query, values, (error) => {
                if (error)
                    return reject(error);
                return resolve();
            });
        });
    });
}
/** @ignore */
function rollback(connection) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            connection.rollback(error => {
                if (error)
                    return reject(error);
                return resolve();
            });
        });
    });
}
