"use strict";
// Copyright (c) 2020 Brandon Lehmann
//
// Please see the included LICENSE file for more information.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Postgres = void 0;
const events_1 = require("events");
const pg_1 = require("pg");
const pgformat = require("pg-format");
const Types_1 = require("./Types");
class Postgres extends events_1.EventEmitter {
    /**
     * Creates a new instance of the class
     * @param host
     * @param port
     * @param username
     * @param password
     * @param database
     */
    constructor(host, port, username, password, database) {
        super();
        this.m_tableOptions = undefined;
        this.m_pool = new pg_1.Pool({
            user: username,
            password: password,
            host: host,
            port: port,
            database: database,
            ssl: {
                rejectUnauthorized: false
            }
        });
        this.m_pool.on('connect', client => this.emit('connect', client));
        this.m_pool.on('acquire', client => this.emit('acquire', client));
        this.m_pool.on('remove', client => this.emit('remove', client));
        this.m_pool.on('error', (error, client) => this.emit('error', error, client));
        this.emit('ready');
    }
    get hashType() {
        return 'char(64)';
    }
    get blobType() {
        return 'text';
    }
    get uint32Type() {
        return 'numeric(10)';
    }
    get uint64Type() {
        return 'numeric(20)';
    }
    get tableOptions() {
        return this.m_tableOptions;
    }
    set tableOptions(value) {
        this.m_tableOptions = value;
    }
    get type() {
        return Types_1.Interfaces.DBType.POSTGRES;
    }
    on(event, listener) {
        return super.on(event, listener);
    }
    close() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.m_pool.end();
        });
    }
    createTable(name, fields, primaryKey, tableOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            const preparedTable = this.prepareCreateTable(name, fields, primaryKey, tableOptions);
            yield this.transaction([
                { query: preparedTable.table }
            ]);
            if (preparedTable.indexes.length !== 0)
                try {
                    const stmts = preparedTable.indexes.map(idx => {
                        return { query: idx };
                    });
                    yield this.transaction(stmts);
                }
                catch (error) {
                    this.emit('error', error);
                }
        });
    }
    prepareCreateTable(name, fields, primaryKey, tableOptions) {
        return Types_1.prepareCreateTable(this.type, name, fields, primaryKey, tableOptions);
    }
    query(query, values, openClient) {
        return __awaiter(this, void 0, void 0, function* () {
            const client = openClient || (yield this.m_pool.connect());
            query = transformQuery(query);
            const res = yield client.query(query, values);
            if (!openClient)
                yield client.release();
            return [res.rowCount, res.rows];
        });
    }
    transaction(queries) {
        return __awaiter(this, void 0, void 0, function* () {
            const client = yield this.m_pool.connect();
            try {
                yield client.query('BEGIN');
                for (const q of queries) {
                    yield this.query(q.query, q.values, client);
                }
                yield client.query('COMMIT');
            }
            catch (e) {
                yield client.query('ROLLBACK');
                throw e;
            }
            finally {
                yield client.release();
            }
        });
    }
    prepareMultiInsert(query, values) {
        return pgformat(query, values);
    }
}
exports.Postgres = Postgres;
/** @ignore */
function transformQuery(query) {
    let counter = 1;
    while (query.indexOf('?') !== -1) {
        query = query.replace(/\?/, `$${counter}`);
        counter++;
    }
    return query;
}
