"use strict";
// Copyright (c) 2020 Brandon Lehmann
//
// Please see the included LICENSE file for more information.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SQLite = void 0;
const events_1 = require("events");
const Types_1 = require("./Types");
const path_1 = require("path");
const sqlite3_1 = require("sqlite3");
const mysql_1 = require("mysql");
/** @ignore */
const pragmaFunctionCalls = [
    'quick_check',
    'integrity_check',
    'incremental_vacuum',
    'foreign_key_check',
    'foreign_key_list',
    'index_info',
    'index_list',
    'index_xinfo',
    'table_info',
    'table_xinfo',
    'optimize'
];
class SQLite extends events_1.EventEmitter {
    /**
     * Creates a new instance of the class
     * @param filepath
     * @param mode
     */
    constructor(filepath, mode = sqlite3_1.OPEN_READWRITE | sqlite3_1.OPEN_CREATE) {
        super();
        this.m_tableOptions = undefined;
        filepath = path_1.resolve(filepath);
        this.m_db = new sqlite3_1.Database(filepath, error => {
            if (error)
                return this.emit('error', error);
            this.emit('ready', filepath, mode);
        });
        this.on('ready', () => __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.setPragma('foreign_keys', true);
            }
            catch (e) {
                this.emit('error', e);
            }
        }));
    }
    get hashType() {
        return 'varchar(64)';
    }
    get blobType() {
        return 'text';
    }
    get uint32Type() {
        return 'unsigned int';
    }
    get uint64Type() {
        return 'unsigned bigint';
    }
    get tableOptions() {
        return this.m_tableOptions;
    }
    set tableOptions(value) {
        this.m_tableOptions = value;
    }
    get type() {
        return Types_1.Interfaces.DBType.SQLITE;
    }
    on(event, listener) {
        return super.on(event, listener);
    }
    createTable(name, fields, primaryKey, tableOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            const preparedTable = this.prepareCreateTable(name, fields, primaryKey, tableOptions);
            yield this.transaction([
                { query: preparedTable.table }
            ]);
            if (preparedTable.indexes.length !== 0)
                try {
                    const stmts = preparedTable.indexes.map(idx => {
                        return { query: idx };
                    });
                    yield this.transaction(stmts);
                }
                catch (error) {
                    this.emit('error', error);
                }
        });
    }
    prepareCreateTable(name, fields, primaryKey, tableOptions) {
        return Types_1.prepareCreateTable(this.type, name, fields, primaryKey, tableOptions);
    }
    getPragma(option) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                option = option.toLowerCase();
                let query = 'PRAGMA ' + option;
                this.m_db.all(query, (error, rows) => {
                    if (error)
                        return reject(error);
                    if (rows.length === 1) {
                        return resolve(rows[0][option]);
                    }
                    else {
                        if (rows[0][option]) {
                            return resolve(rows.map(elem => elem[option]));
                        }
                    }
                    return resolve(rows);
                });
            });
        });
    }
    setPragma(option, value) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                option = option.toLowerCase();
                let query = 'PRAGMA ';
                if (pragmaFunctionCalls.indexOf(option) !== -1) {
                    query += option + '(' + value + ')';
                }
                else {
                    query += option + ' = ' + value;
                }
                this.m_db.run(query, error => {
                    if (error)
                        return reject(error);
                    return resolve();
                });
            });
        });
    }
    close() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                this.m_db.close(error => {
                    if (error)
                        return reject(error);
                    return resolve();
                });
            });
        });
    }
    query(query, values) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                if (query.toLowerCase().startsWith('select')) {
                    this.m_db.all(query, values, (err, rows) => {
                        if (err)
                            return reject(err);
                        return resolve([rows.length, rows]);
                    });
                }
                else {
                    this.m_db.run(query, values, function (err) {
                        if (err)
                            return reject(err);
                        const count = (query.toLowerCase().startsWith('insert')) ? this.lastID :
                            (query.toLowerCase().startsWith('update')
                                || query.toLowerCase().startsWith('delete')) ? this.changes : 0;
                        return resolve([count, []]);
                    });
                }
            });
        });
    }
    connection() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.m_db;
        });
    }
    transaction(queries) {
        return __awaiter(this, void 0, void 0, function* () {
            const connection = yield this.connection();
            try {
                yield beginTransaction(connection);
                for (const q of queries) {
                    yield query(connection, q.query, q.values);
                }
                yield commit(connection);
            }
            catch (e) {
                yield rollback(connection);
                throw e;
            }
        });
    }
    prepareMultiInsert(query, values) {
        if (values) {
            const escaped = mysql_1.escape(values);
            return query.replace('%L', escaped);
        }
        return query;
    }
}
exports.SQLite = SQLite;
/** @ignore */
function beginTransaction(connection) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            connection.run('BEGIN TRANSACTION', err => {
                if (err)
                    return reject(err);
                return resolve();
            });
        });
    });
}
/** @ignore */
function commit(connection) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            connection.run('COMMIT', err => {
                if (err)
                    return reject(err);
                return resolve();
            });
        });
    });
}
/** @ignore */
function query(connection, query, values = []) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            connection.run(query, values, err => {
                if (err)
                    return reject(err);
                return resolve();
            });
        });
    });
}
/** @ignore */
function rollback(connection) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            connection.run('ROLLBACK', err => {
                if (err)
                    return reject(err);
                return resolve();
            });
        });
    });
}
