export declare namespace Interfaces {
    /**
     * The database type
     */
    enum DBType {
        MYSQL = 0,
        POSTGRES = 1,
        SQLITE = 2
    }
    /**
     * Bulk Query interface that allows us to specify a single query with
     * a multidimensional array of values
     */
    interface IBulkQuery {
        query: string;
        values?: any[];
    }
    /**
     * Column interface that represents an individual table column
     */
    interface ITableColumn {
        name: string;
        type: string;
        nullable?: boolean;
        foreign?: IForeignKey;
        unique?: boolean;
        default?: string | number;
    }
    /**
     * Foreign Key interface that represents a foreign key relationship
     * with a defined constraint
     */
    interface IForeignKey {
        table: string;
        column: string;
        update?: string;
        delete?: string;
    }
    /**
     * Foreign key constraint action enum
     */
    enum FKAction {
        RESTRICT = "RESTRICT",
        CASCADE = "CASCADE",
        NULL = "SET NULL",
        DEFAULT = "SET DEFAULT",
        NA = "NO ACTION"
    }
    /**
     * Represents a query result such that [0] is the number of rows returned,
     * affected, or changed, depending on the query verb
     */
    type IQueryResult = [number, any[]];
    /**
     * A nested array of values
     */
    type IValueArray = any[][];
}
/**
 * Abstract database class that all underlying database
 * classes must implement so that we have consistency in methods
 */
export declare abstract class IDatabase {
    /**
     * Returns the column type for the DB for a hash (ie. char(64))
     */
    abstract get hashType(): string;
    /**
     * Returns the column type for the DB to store a hexadecimal blob of data
     */
    abstract get blobType(): string;
    /**
     * Returns the column type for the DB to store a uint32_t value
     */
    abstract get uint32Type(): string;
    /**
     * Returns the column type for the DB to store a uint64_t value
     */
    abstract get uint64Type(): string;
    /**
     * Returns a string of additional options that need to be applied to tables
     * which may include ENGINE information, compression, etc depending on the DB system
     */
    abstract get tableOptions(): string | undefined;
    /**
     * Returns the database type
     */
    abstract get type(): Interfaces.DBType;
    /**
     * Creates the table with the given fields and options
     * @param name
     * @param fields
     * @param primaryKey
     * @param tableOptions
     */
    abstract createTable(name: string, fields: Interfaces.ITableColumn[], primaryKey: string[], tableOptions?: string): Promise<void>;
    /**
     * Creates the SQL statements necessary to create a table with the given
     * fields and options
     * @param name
     * @param fields
     * @param primaryKey
     * @param tableOptions
     */
    abstract prepareCreateTable(name: string, fields: Interfaces.ITableColumn[], primaryKey: string[], tableOptions?: string): {
        table: string;
        indexes: string[];
    };
    /**
     * Closes the database connection(s)
     */
    abstract close(): Promise<void>;
    /**
     * Performs a query and returns the result
     * @param query
     * @param values
     */
    abstract query(query: string, values?: any[]): Promise<Interfaces.IQueryResult>;
    /**
     * Constructs and executes the given queries in a transaction
     * @param queries
     */
    abstract transaction(queries: Interfaces.IBulkQuery[]): Promise<void>;
    /**
     * Prepares a query to perform a multi-insert statement which is
     * far faster than a whole bunch of individual insert statements
     * @param query
     * @param values
     */
    abstract prepareMultiInsert(query: string, values?: Interfaces.IValueArray): string;
}
export declare function prepareCreateTable(dbType: Interfaces.DBType, name: string, fields: Interfaces.ITableColumn[], primaryKey: string[], tableOptions: string | undefined): {
    table: string;
    indexes: string[];
};
